'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash.clonedeep');

var _lodash2 = _interopRequireDefault(_lodash);

var _constants = require('./constants');

var c = _interopRequireWildcard(_constants);

var _pages = require('./migrations/pages');

var pages = _interopRequireWildcard(_pages);

var _sections = require('./migrations/sections');

var sections = _interopRequireWildcard(_sections);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Content {
  constructor(content, structure) {
    this.content = _extends({
      global: {},
      pages: {},
      sections: {}
    }, (0, _lodash2.default)(content));
    this.structure = _extends({
      global: {},
      pages: {},
      sections: {},
      fields: {}
    }, (0, _lodash2.default)(structure));
  }

  getContent() {
    return this.content;
  }

  _addPages(newPages) {
    newPages.forEach(page => {
      this.content.global._items.push(page._id);
      this.content.pages[page._id] = page;
    });
  }

  _addSections(newSections, page) {
    newSections.forEach(section => {
      page._items.push(section._id);
      this.content.sections[section._id] = section;
    });
  }

  _addMissingFields(validFields) {
    let itemStructure = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    let itemContent = arguments[2];
    var _itemStructure$fields = itemStructure.fields;
    const fields = _itemStructure$fields === undefined ? [] : _itemStructure$fields;


    const itemFields = Object.keys(itemContent).filter(prop => !prop.startsWith('_'));

    const missingItemFields = validFields.filter(field => fields.includes(field) && !itemFields.includes(field));

    missingItemFields.forEach(fieldName => {
      if (this.structure.fields[fieldName].type === 'list') {
        itemContent[fieldName] = [];
      } else {
        itemContent[fieldName] = '';
      }
      itemFields.push(fieldName);
    });

    itemFields.forEach(fieldName => {
      const fieldStructure = this.structure.fields[fieldName];
      if (fieldStructure.type === 'list') {
        while (itemContent[fieldName].length < fieldStructure.minItems) {
          itemContent[fieldName].push({});
        }

        itemContent[fieldName].forEach(listContent => {
          this._addMissingFields(validFields, this.structure.fields[fieldName], listContent);
        });
      }
    });
  }

  _removePages(pageIdsToRemove) {
    pageIdsToRemove.forEach(id => {
      this.content.global._items = this.content.global._items.filter(pageId => pageId !== id);

      delete this.content.pages[id];
    });
  }

  _removeSections(sectionIdsToRemove) {
    sectionIdsToRemove.forEach(id => {
      Object.values(this.content.pages).forEach(page => {
        page._items = page._items.filter(sectionId => sectionId !== id);
      });

      delete this.content.sections[id];
    });
  }

  _removeInvalidFields(validFields) {
    let itemStructure = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    let itemContent = arguments[2];
    var _itemStructure$fields2 = itemStructure.fields;
    const fields = _itemStructure$fields2 === undefined ? [] : _itemStructure$fields2;


    Object.keys(itemContent).filter(prop => !prop.startsWith('_')).forEach(fieldName => {
      const validItemFields = validFields.filter(field => fields.includes(field));

      if (validItemFields.includes(fieldName)) {
        const fieldStructure = this.structure.fields[fieldName];
        if (fieldStructure.type === 'list') {
          while (itemContent[fieldName].length > fieldStructure.maxItems) {
            itemContent[fieldName].pop();
          }

          itemContent[fieldName].forEach(listContent => {
            this._removeInvalidFields(validFields, fieldStructure, listContent);
          });
        }
      } else {
        delete itemContent[fieldName];
      }
    });
  }

  migratePages() {
    this.content.global._items = this.content.global._items || [];

    const invalidPageIds = pages.getInvalidPageIds(this.structure.pages, this.content.pages);
    this._removePages(invalidPageIds);

    const requiredPages = pages.getRequiredPages(this.structure, this.content.pages);
    this._addPages(requiredPages);

    const pagesOverMaximum = pages.getPagesOverMaximum(this.structure, this.content.pages);
    this._removePages(pagesOverMaximum);

    return this;
  }

  migrateSections() {
    const invalidSectionIds = sections.getInvalidSectionIds(this.structure, this.content);
    this._removeSections(invalidSectionIds);

    Object.values(this.content.pages).forEach(page => {
      const requiredSections = sections.getRequiredSectionsForPage(this.structure.pages, page);
      this._addSections(requiredSections, page);

      const sectionsOverMaximum = sections.getSectionsOverMaximum(this.structure.pages, page);
      this._removeSections(sectionsOverMaximum);
    });

    return this;
  }

  migrateFields() {
    const validFields = Object.keys(this.structure.fields);

    this._removeInvalidFields(validFields, this.structure.global, this.content.global);
    this._addMissingFields(validFields, this.structure.global, this.content.global);

    Object.values(this.content.pages).forEach(page => {
      const pageStructure = this.structure.pages[page._type];
      const pageContent = this.content.pages[page._id];
      this._removeInvalidFields(validFields, pageStructure, pageContent);
      this._addMissingFields(validFields, pageStructure, pageContent);
    });

    Object.values(this.content.sections).forEach(section => {
      const sectionStructure = this.structure.sections[section._type];
      const sectionContent = this.content.sections[section._id];
      this._removeInvalidFields(validFields, sectionStructure, sectionContent);
      this._addMissingFields(validFields, sectionStructure, sectionContent);
    });

    return this;
  }
}
exports.default = Content;