'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRequiredPages = exports.getPagesOverMaximum = exports.getInvalidPageIds = exports._getProtectedPageTypes = exports._createPage = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _constants = require('../constants');

var c = _interopRequireWildcard(_constants);

var _utils = require('../utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const _createPage = exports._createPage = function _createPage() {
  let _type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : c.TYPE_STANDARD;

  return {
    _id: (0, _utils.getContentId)(),
    _items: [],
    _type: _type
  };
};

const _getProtectedPageTypes = exports._getProtectedPageTypes = pages => Object.entries(pages).reduce((acc, _ref) => {
  var _ref2 = _slicedToArray(_ref, 2);

  let type = _ref2[0],
      page = _ref2[1];

  if (page.isProtected) acc.push(type);
  return acc;
}, []);

const getInvalidPageIds = exports.getInvalidPageIds = (pagesStructure, pagesContent) => {
  const invalidPageIds = [];
  const validPageTypes = [...Object.keys(pagesStructure), c.TYPE_STANDARD];

  Object.values(pagesContent).forEach(page => {
    if (!validPageTypes.includes(page._type)) {
      invalidPageIds.push(page._id);
    }
  });

  return invalidPageIds;
};

const getPagesOverMaximum = exports.getPagesOverMaximum = (structure, pagesContent) => {
  const pagesOverMaximum = [];
  const protectedPageTypes = _getProtectedPageTypes(structure.pages);
  const maxAmountOfPages = structure.global.maxItems;
  const existingPagesArray = Object.values(pagesContent).reverse();

  while (existingPagesArray.length > maxAmountOfPages) {
    const pageIndex = existingPagesArray.findIndex((_ref3) => {
      let _type = _ref3._type;
      return !protectedPageTypes.includes(_type);
    });

    if (pageIndex !== -1) {
      var _existingPagesArray$s = existingPagesArray.splice(pageIndex, 1),
          _existingPagesArray$s2 = _slicedToArray(_existingPagesArray$s, 1);

      const pageToRemove = _existingPagesArray$s2[0];

      pagesOverMaximum.push(pageToRemove._id);
    } else {
      break;
    }
  }

  return pagesOverMaximum;
};

const getRequiredPages = exports.getRequiredPages = (structure, pagesContent) => {
  const requiredPages = [];
  const protectedPageTypes = _getProtectedPageTypes(structure.pages);
  const minAmountOfPages = structure.global.minItems;
  const existingPagesArray = Object.values(pagesContent);

  protectedPageTypes.forEach(type => {
    const hasType = existingPagesArray.some(page => type === page._type);
    if (!hasType) {
      requiredPages.push(_createPage(type));
    }
  });

  let totalAmountOfPages = existingPagesArray.length + requiredPages.length;
  while (totalAmountOfPages < minAmountOfPages) {
    requiredPages.push(_createPage());
    totalAmountOfPages += 1;
  }

  return requiredPages;
};