'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSectionsOverMaximum = exports.getRequiredSectionsForPage = exports.getInvalidSectionIds = exports._createSection = undefined;

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

var _constants = require('../constants');

var c = _interopRequireWildcard(_constants);

var _utils = require('../utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const _createSection = exports._createSection = type => ({
  _id: (0, _utils.getContentId)(),
  _type: type || c.TYPE_STANDARD
});

const getInvalidSectionIds = exports.getInvalidSectionIds = (structure, content) => {
  let invalidSectionIds = [];
  const validSectionTypes = Object.keys(structure.sections);

  Object.values(content.pages).forEach(page => {
    const validSectionTypesForPage = structure.global.maxItems > 1 ? structure.pages[page._type].allowedItems || [c.TYPE_STANDARD] : validSectionTypes;

    const invalidSectionIdsForPage = page._items.filter(sectionId => {
      const sectionType = content.sections[sectionId]._type;

      return !(validSectionTypes.includes(sectionType) && validSectionTypesForPage.includes(sectionType));
    });
    invalidSectionIds = [...invalidSectionIds, ...invalidSectionIdsForPage];
  });

  return invalidSectionIds;
};

const getRequiredSectionsForPage = exports.getRequiredSectionsForPage = (pagesStructure, page) => {
  const requiredSections = [];
  const allowedSectionTypes = (0, _lodash2.default)(pagesStructure, `${page._type}.allowedItems`, [c.TYPE_STANDARD]);
  const minSections = (0, _lodash2.default)(pagesStructure, `${page._type}.minItems`, 0);

  let totalAmountOfSections = page._items.length;
  while (totalAmountOfSections < minSections) {
    requiredSections.push(_createSection(allowedSectionTypes[0]));
    totalAmountOfSections += 1;
  }

  return requiredSections;
};

const getSectionsOverMaximum = exports.getSectionsOverMaximum = (pagesStructure, page) => {
  const maxSections = (0, _lodash2.default)(pagesStructure, `${page._type}.maxItems`, Infinity);

  return page._items.slice(maxSections);
};