'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readJSON = readJSON;
exports.getContentId = getContentId;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function readJSON(path, createIfNotExists) {
  try {
    const file = _fs2.default.readFileSync(path);
    return {
      error: false,
      file: JSON.parse(file.toString())
    };
  } catch (error) {
    if (error.code === 'ENOENT') {
      if (createIfNotExists) {
        _fs2.default.writeFileSync(path, '{}');
        return {
          file: {}
        };
      }

      return {
        error: `File not found. Are you sure it exists? [${path}]`
      };
    } else if (error.toString().includes('SyntaxError')) {
      return {
        error: `Couldn\'t parse file. Please check for a valid JSON format. [${path}]`
      };
    }

    return {
      error: 'Couldn\'t read file.'
    };
  }
}

function getContentId() {
  return Math.random().toString(36).substring(2, 9);
}